//
//  BFVoiceRecordManager.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2021/11/24.
//

import BFCommonKit
import BFMediaKit
import Foundation

class BFVoiceRecordManager: NSObject {
    // 录音相关
    var audioRecorder: BFRecorderManager?
    //录音结束回调
    var endRecordHandle: ((PQVoiceModel?, Error?) -> Void)?
    //录音取消回调
    var cancelRecordHandle: ((PQVoiceModel?) -> Void)?
    //录音进度回调
    var recorderProgrossHandle: ((Float64?) -> Void)?
    //字幕的回调 参数1： 字幕数据 ，参数2 ：对应的录音文件
    var subtitleRecordHandle: ((String?,String?) -> Void)?
    //开始录制时间
    var beginRecordTime: Date = Date()
    //音频文件模型
    var voiceModel: PQVoiceModel?
    //停止是否为取消操作
    var mIsCancel:Bool = false
    
    /// 初始化方法
    /// - Parameters:
    ///   - token: NLS token
    ///   - appid: NLS appid
    public init(token: String, appid: String) {
        super.init()
        audioRecorder = BFRecorderManager(token, appid: appid)
        audioRecorder?.delegate = self
    }
    
    
    /// 开始录音
    func startRecord() {
        var recorderFilePath = exportAudiosDirectory
        
        if !directoryIsExists(dicPath: recorderFilePath) {
            BFLog(message: "文件夹不存在 \(recorderFilePath)")
            createDirectory(path: recorderFilePath)
        }
        recorderFilePath.append("recorder_\(Date().timeIntervalSince1970).pcm")
        BFLog(1, message: "开始录音 \(recorderFilePath)")
        
        BFLog(1, message: "开始录制")
        audioRecorder?.startRecord(recorderFilePath)
        beginRecordTime = Date()
    }
    
    /// 停止录制
    /// - Parameter isCancel: 是否为取消 ,取消操作会把录制的文件删除和字幕删除
    func stopRecord(isCancel: Bool) {
        
        mIsCancel = isCancel
        audioRecorder?.stopRecord()
    }
}

// MARK: - 录音机回调
extension BFVoiceRecordManager: BFRecorderManagerDelegate {
    public func recorderProgress(_: BFRecorderManager, recoderTime: Double) {
        BFLog(message: "录音机进度:\(recoderTime)")
        recorderProgrossHandle?(recoderTime)
    }
    
    public func recorderDidStop(_ outfile: String) {
        let duration = Date().timeIntervalSince(beginRecordTime)
        if duration > 1 {
            
            if(mIsCancel){
                // 删除录制的原文件
                do {
                    try FileManager.default.removeItem(atPath: outfile)
                    print("取消后删除文件Success to remove recorder file. \(outfile)")
                } catch {
                    print("取消后删除文件Failed to remove recorder file. \(outfile)")
                }
                cancelRecordHandle?(nil)
            }else{
                
                var  beginRecordTime1 = Date()
               
                //1转wav
                let wavFilePath = outfile.replacingOccurrences(of: ".pcm", with: ".wav")
                BFPcmToWaveTool().pcmToWav(inFileName: outfile, outFileName: wavFilePath)
                BFLog(message: "转 WAV用时\( Date().timeIntervalSince(beginRecordTime1))")
                // 删除录制的pcm文件
                do {
                    try FileManager.default.removeItem(atPath: outfile)
                    print("Success to remove recorder file. \(outfile)")
                } catch {
                    print("Failed to remove recorder file. \(outfile)")
                }
                
                //2处理降噪
                beginRecordTime1 = Date()
                let noiseFilePath = wavFilePath.replacingOccurrences(of: ".wav", with: "_noise.wav")
                BFLog(1, message: "降噪后地址：\(noiseFilePath) 原地址：\(wavFilePath)")
                NXNoiseReduction().denoise(wavFilePath, outFile: noiseFilePath)
                if let model = voiceModel {
                    model.wavFilePath = noiseFilePath
                    model.duration = "\(duration)"
                    endRecordHandle?(model, nil)
                    
                    BFLog(message: "降噪用时\( Date().timeIntervalSince(beginRecordTime1))")
                    
                }
                // 删除临时 wav 文件
                do {
                    try FileManager.default.removeItem(atPath: wavFilePath)
                    print("Success to remove recorder file. \(wavFilePath)")
                } catch {
                    print("Failed to remove recorder file. \(wavFilePath)")
                }
                

            }
            
        } else {
            cShowHUB(superView: nil, msg: "最短录制1秒")
            cancelRecordHandle?(voiceModel)
        }
    }
    
    public func eventCallback(_: BFRecorderManager, asrResult: String,audioFilePath:String) {
        //最后输出的文件是降噪后的
        let noiseFilePath = audioFilePath.replacingOccurrences(of: ".pcm", with: "_noise.wav")
        subtitleRecordHandle?(asrResult, noiseFilePath)
    }
}
